import 'package:flutter/material.dart';

class SizeReportingWidget extends StatefulWidget {
  Widget child;
  ValueChanged<Size> onSizeChange;
  bool shouldRefresh;

  SizeReportingWidget({
    Key? key,
    required this.child,
    required this.onSizeChange,
    this.shouldRefresh = false,
  }) : super(key: key);

  @override
  _SizeReportingWidgetState createState() => _SizeReportingWidgetState();
}

class _SizeReportingWidgetState extends State<SizeReportingWidget> {
  final _widgetKey = GlobalKey();
  Size? _oldSize;

  @override
  Widget build(BuildContext context) {
    WidgetsBinding.instance!.addPostFrameCallback((_) => _notifySize());
    return NotificationListener<SizeChangedLayoutNotification>(
      onNotification: (_) {
        WidgetsBinding.instance!.addPostFrameCallback((_) => _notifySize());
        return true;
      },
      child: SizeChangedLayoutNotifier(
        child: Container(
          key: _widgetKey,
          child: widget.child,
        ),
      ),
    );
  }

  void _notifySize() {
    final context = _widgetKey.currentContext;
    if (context == null) return;
    final size = context.size;
    if (_oldSize != size || widget.shouldRefresh) {
      _oldSize = size;
      widget.onSizeChange(size!);
    }
  }
}
